use serde::Serialize;
use serde_json::{Value, json};
#[macro_use]
mod modules;
use crate::modules::{cpu, devices, external, gpu, memory, meta, network, os, storage, uptime};
use anyhow::Result;

#[derive(Debug, Serialize)]
pub struct TelemetryInfo {
    telemetry: AllInfo,
}

#[derive(Debug, Serialize)]
pub struct AllInfo {
    gpu: gpu::GPUInfo,
    cpu: cpu::CPUInfo,
    memory: memory::MemoryInfo,
    os: os::OSInfo,
    storage: Vec<storage::StorageInfo>,
    network: network::NetworkInfo,
    external: external::ExternalInfo,
    devices: devices::DevicesInfo,
    meta: meta::MetaInfo,
    uptime: uptime::UptimeInfo,
}

#[allow(dead_code)]
pub fn get_struct() -> Result<TelemetryInfo> {
    Ok(TelemetryInfo {
        telemetry: AllInfo {
            gpu: gpu::get_struct(),
            cpu: cpu::get_struct(),
            memory: memory::get_struct(),
            os: os::get_struct()?,
            storage: storage::get_list()?,
            network: network::get_struct(),
            external: external::get_struct(),
            devices: devices::get_struct()?,
            meta: meta::get_struct()?,
            uptime: uptime::get_struct()?,
        },
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value> {
    Ok(json!(get_struct()?))
}
