use super::Defaults::Unknown;
use serde::Serialize;
use serde_json::{Value, json};
use std::collections::HashMap;
use sysinfo::{CpuRefreshKind, RefreshKind, System};

#[derive(Debug, Serialize)]
pub struct ProcessorInfo {
    vendor: String,
    brand: String,
    total_system_cores: String,
    threads: u16,
    architecture: String,
    byte_order: String,
}

#[derive(Debug, Serialize)]
pub struct CPUInfo {
    cpus: Vec<ProcessorInfo>,
}

#[allow(dead_code)]
pub fn get_struct() -> CPUInfo {
    let system =
        System::new_with_specifics(RefreshKind::nothing().with_cpu(CpuRefreshKind::everything()));

    let mut processors = Vec::new();
    let mut processor_count: HashMap<String, ProcessorInfo> = HashMap::new();

    for cpu in system.cpus() {
        let brand = cpu.brand().trim();
        let total_cores = System::physical_core_count();

        if processor_count.contains_key(brand) {
            processor_count.get_mut(brand).unwrap().threads += 1;
            continue;
        }
        processor_count.insert(
            brand.to_string(),
            ProcessorInfo {
                vendor: cpu.vendor_id().trim().to_string(),
                brand: brand.to_string(),
                total_system_cores: if let Some(cores) = total_cores {
                    cores.to_string()
                } else {
                    Unknown.into()
                },
                threads: 1,
                architecture: System::cpu_arch().trim().to_string(),
                byte_order: if cfg!(target_endian = "little") {
                    String::from("little-endian")
                } else {
                    String::from("big-endian")
                },
            },
        );
    }

    for (_brand, info) in processor_count {
        processors.push(info);
    }

    CPUInfo { cpus: processors }
}

#[allow(dead_code)]
pub fn get_json() -> Value {
    json!(get_struct())
}
