//use crate::custom_anyhow;
use anyhow::Result;
use serde::Serialize;
use serde_json::{Value, json};
extern crate hidapi;
use hidapi::HidApi;
use std::collections::HashMap;

#[derive(Debug, Serialize)]
pub struct DeviceInfo {
    manufacturer: String,
    products: Vec<String>,
}

#[derive(Debug, Serialize)]
pub struct DevicesInfo {
    devices: Vec<DeviceInfo>,
}

#[allow(dead_code)]
pub fn get_struct() -> Result<DevicesInfo> {
    let api = HidApi::new().unwrap();
    let mut grouped_devices: HashMap<String, Vec<String>> = HashMap::new();

    for device in api.device_list() {
        let manufacturer = device
            .manufacturer_string()
            .unwrap_or("Unknown")
            .to_string()
            .to_lowercase();
        let product = device.product_string().unwrap_or("Unknown").to_string();

        if !manufacturer.trim().is_empty() && !product.trim().is_empty() {
            let entry = grouped_devices.entry(manufacturer).or_default();

            if !entry.contains(&product) {
                entry.push(product);
            }
        }
    }

    let mut devices = Vec::new();
    let mut grouped_vec: Vec<_> = grouped_devices.into_iter().collect();
    grouped_vec.sort_by(|a, b| a.0.to_lowercase().cmp(&b.0.to_lowercase()));

    for (manufacturer, products) in grouped_vec {
        devices.push(DeviceInfo {
            manufacturer,
            products,
        });
    }

    Ok(DevicesInfo { devices })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value> {
    Ok(json!(get_struct()?))
}
