use serde::Serialize;
use std::{
    fmt::{Debug, Display, Formatter, Result as FmtResult},
    ops::Deref,
};

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Debug, Serialize)]
pub enum TargetPointerWidth {
    W32Bit,
    W64Bit,
}

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct FmtOSArchitecture(pub TargetPointerWidth);

impl FmtOSArchitecture {
    pub const W32_BIT: Self = FmtOSArchitecture(TargetPointerWidth::W32Bit);
    pub const W64_BIT: Self = FmtOSArchitecture(TargetPointerWidth::W64Bit);
}

impl Display for FmtOSArchitecture {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        write!(
            f,
            "{}",
            match self.0 {
                TargetPointerWidth::W32Bit => "32 Bit",
                TargetPointerWidth::W64Bit => "64 Bit",
            }
        )
    }
}

impl Debug for FmtOSArchitecture {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtOSArchitecture {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl Deref for FmtOSArchitecture {
    type Target = TargetPointerWidth;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
