use serde::Serialize;
use std::{
    fmt::{Debug, Display, Formatter, Result as FmtResult},
    ops::Deref,
};

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct FmtBytes(pub u64);

impl Display for FmtBytes {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        use humansize::{DECIMAL, format_size};
        write!(f, "{}", format_size(self.0, DECIMAL))
    }
}

impl Debug for FmtBytes {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtBytes {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl Deref for FmtBytes {
    type Target = u64;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
