use detect_desktop_environment::DesktopEnvironment;
use serde::{Serialize, Serializer};
use std::{
    fmt::{Debug, Display, Formatter, Result as FmtResult},
    ops::Deref,
};

#[derive(Copy, Clone, PartialEq, Eq, Debug)]
pub struct SerdeDE(pub DesktopEnvironment);

impl Serialize for SerdeDE {
    fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
        serializer.serialize_str(&format!("{:?}", self.0))
    }
}

impl SerdeDE {
    pub fn detect() -> Option<Self> {
        DesktopEnvironment::detect().map(SerdeDE)
    }
}

#[derive(Copy, Clone, PartialEq, Eq)]
pub struct FmtDE(pub SerdeDE);

impl FmtDE {
    pub fn detect() -> Option<Self> {
        SerdeDE::detect().map(FmtDE)
    }
}

impl Display for FmtDE {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        write!(f, "{:?}", self.0.0)
    }
}

impl Debug for FmtDE {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtDE {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl Deref for FmtDE {
    type Target = SerdeDE;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
