use serde::Serialize;
use std::{
    fmt::{Debug, Display, Formatter, Result as FmtResult},
    ops::Deref,
    time::Duration,
};

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct FmtRelativeTime(pub Duration);

impl Display for FmtRelativeTime {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        let formatter = timeago::Formatter::new();

        write!(f, "{}", formatter.convert(self.0))
    }
}

impl Debug for FmtRelativeTime {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtRelativeTime {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl Deref for FmtRelativeTime {
    type Target = Duration;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
