use serde::Serialize;
use std::{
    fmt::{Debug, Display, Formatter, Result as FmtResult},
    ops::Deref,
};
use versions::Versioning;

#[derive(Clone, PartialEq, Eq, PartialOrd, Ord)]
pub struct FmtVersion(pub Versioning);

impl Display for FmtVersion {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        write!(f, "{}", self.0)
    }
}

impl Debug for FmtVersion {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtVersion {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: serde::Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl Deref for FmtVersion {
    type Target = Versioning;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
