use crate::modules::FmtBytes;
use serde::Serialize;
use serde_json::{Value, json};
use sysinfo::System;

#[derive(Debug, Serialize)]
pub struct PhysicalInfo {
    total: FmtBytes,
    used: FmtBytes,
    free: FmtBytes,
    available: FmtBytes,
}

#[derive(Debug, Serialize)]
pub struct VirtualInfo {
    total: FmtBytes,
    used: FmtBytes,
    free: FmtBytes,
}

#[derive(Debug, Serialize)]
pub struct MemoryInfo {
    physical: PhysicalInfo,
    virtual_swap: VirtualInfo,
}

#[allow(dead_code)]
pub fn get_struct() -> MemoryInfo {
    let system = System::new_all();

    MemoryInfo {
        physical: PhysicalInfo {
            total: FmtBytes(system.total_memory()),
            used: FmtBytes(system.used_memory()),
            free: FmtBytes(system.free_memory()),
            available: FmtBytes(system.available_memory()),
        },
        virtual_swap: VirtualInfo {
            total: FmtBytes(system.total_swap()),
            used: FmtBytes(system.used_swap()),
            free: FmtBytes(system.free_swap()),
        },
    }
}

#[allow(dead_code)]
pub fn get_json() -> Value {
    json!(get_struct())
}
