use crate::{custom_anyhow, modules::FmtVersion};
use anyhow::Result;
use serde::Serialize;
use serde_json::{Value, json};
use sys_locale::get_locale;
use versions::Versioning;

include!(concat!(env!("OUT_DIR"), "/built.rs"));

#[derive(Debug, Serialize)]
pub struct MetaInfo {
    version: FmtVersion,
    commit_hash: String,
    locale: String,
}

#[allow(dead_code)]
pub fn get_struct() -> Result<MetaInfo> {
    let locale = get_locale().unwrap_or_else(|| String::from("en-US"));
    let version = Versioning::new(env!("CARGO_PKG_VERSION"))
        .ok_or(custom_anyhow!("Invalid version").context("Invalid Semver version"))?;

    Ok(MetaInfo {
        version: FmtVersion(version),
        commit_hash: GIT_COMMIT_HASH.to_string(),
        locale,
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value> {
    Ok(json!(get_struct()?))
}
