use std::fmt::{Debug, Display, Formatter, Result as FmtResult};

pub mod cpu;
pub mod devices;
pub mod external;
pub mod formatted;
pub mod gpu;
pub mod macros;
pub mod memory;
pub mod meta;
pub mod network;
pub mod os;
pub mod storage;
pub mod uptime;

#[allow(unused_imports)]
pub use formatted::{
    architecture::FmtOSArchitecture,
    bytes::FmtBytes,
    date_time::{FmtDateTime, IntoDateTime},
    desktop_environment::FmtDE,
    offset_time::FmtOffsetTime,
    relative_time::FmtRelativeTime,
    version::FmtVersion,
};

#[derive(Debug)]
pub enum Defaults {
    Unknown,
}

impl Display for Defaults {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        write!(f, "{:?}", self)
    }
}

impl From<Defaults> for String {
    fn from(value: Defaults) -> Self {
        value.to_string()
    }
}
