use super::Defaults::Unknown;
use crate::{
    custom_anyhow,
    modules::{FmtDE, FmtOSArchitecture},
};
use anyhow::Result;
use serde::Serialize;
use serde_json::{Value, json};
use sysinfo::System;

#[derive(Debug, Serialize)]
pub struct OSInfo {
    name: String,
    edition: String,
    version: String,
    architecture: FmtOSArchitecture,
    kernel: Option<String>,
    desktop_environment: Option<FmtDE>,
}

#[allow(dead_code)]
pub fn get_struct() -> Result<OSInfo> {
    let architecture = if cfg!(target_pointer_width = "64") {
        Ok(FmtOSArchitecture::W64_BIT)
    } else if cfg!(target_pointer_width = "32") {
        Ok(FmtOSArchitecture::W32_BIT)
    } else {
        Err(custom_anyhow!("Unsupported pointer width").context("16 bit systems are not supported"))
    }?;
    let kernel = if cfg!(target_os = "linux") || cfg!(target_os = "macos") {
        Some(System::kernel_long_version())
    } else {
        None
    };
    let desktop_environment = if cfg!(target_os = "linux") {
        FmtDE::detect()
    } else {
        None
    };

    Ok(OSInfo {
        name: System::name().unwrap_or(Unknown.into()),
        edition: System::long_os_version().unwrap_or(Unknown.into()),
        version: System::os_version().unwrap_or(Unknown.into()),
        architecture,
        kernel,
        desktop_environment,
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value> {
    Ok(json!(get_struct()?))
}
